;;;
;;; GTK Utilities
;;;
;;; Copyright (C) 1998 David Lutterkort <lutter@cs.purdue.edu>
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2, or (at
;;; your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this software; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
;;; USA.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This file defines several convenience functions for using
;; GTK from guile.
;;
;; gtku-widget-show*  : show a whole bunch of widgets at once
;; gtku-make-menu     : create a menu from a describing list
;; gtku-make-menu-bar : create a menu bar from a list menu descriptions
;; gtku-menu-item     : generate a simple callback for a very common case
;; gtku-radio-menu    : generate a radio group + simple callbacks 
;; gtku-yes-or-no?    : display a yes or no dialog box

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Show a whole list of widgets:
;; (gtku-widget-show* widget1 widget2... widgetn) 
;; shows all these widgets
;; Taken from Marius Vollmers calc.scm
(define (gtku-widget-show* . widgets)
  (for-each gtk-widget-show widgets))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu helpers
;;
;; This collection of macros and functions makes it easier to
;; construct GTK menus. The menu can be described by a language
;; (represented as nested lists) that has the following syntax:
;;
;; MENU := (NAME ITEMS)
;; ITEMS := ITEM ITEMS | MENU | empty
;; ITEM  := (NAME . ACTION)               'normal' menu item
;;        | (check NAME . ACTION)         menu item with checkbox
;;        | (radio (NAME . ACTION)+)      list of mutually excl. items
;; NAME  := STRING
;;
;; ACTION is the callback that gets associated with a menu entry's 
;; "activate" signal and has to be a thunk. If no action is to 
;; be associated with an item, ACTION should be #f

(define (gtku-make-menu descr)
  ;; Create a GTK Menu from a MENU description
  ;; Returns gtk-menu-item that contains the whole menu
  ;; and can be stuck in a menu-bar or the like
  (let ((menu (gtk-menu-new))
	(items (cdr descr))
	(title (gtk-menu-item-new-with-label (car descr))))
    
    (define (add-action item thunk)
      ; Connect an "activate" signal to item and add it to menu
      (gtk-menu-append menu item)
      (gtk-widget-show item)
      (if thunk (gtk-signal-connect item "activate" thunk))
      item)

    (define (make-radio-group items)
      ; Add a group of mutually exclusive radio items to the current menu
      ; items is a list with entries of the form (NAME . ACTION)
      (let ((group #f)
	    (radio-item gtk-radio-menu-item-new-with-label))
	(for-each
	 (lambda (x)			; x is (NAME . ACTION)
	   (set! group 
		 (add-action (radio-item group (car x)) (cdr x))))
	 items)
	group))
    
    (define (make-item item)
      (if (not (pair? (cdr item)))	
	  (add-action			; (NAME . ACTION)
	   (gtk-menu-item-new-with-label (car item))
	   (cdr item))
	  (case (car item)
	    ((check)			; (check NAME . ACTION)
	     (add-action
	      (gtk-check-menu-item-new-with-label (cadr item))
	      (cddr item)))
	    ((radio)			; (radio (NAME . ACTION)+)
	     (make-radio-group (cdr item)))
	    (else			; (NAME ITEMS)
	     (add-action (gtku-make-menu item) #f)))))

    (gtk-menu-item-set-submenu title menu)
    (gtk-widget-show title)
    (for-each make-item items)
    title))
    
(define (gtku-make-menu-bar bar)
  ;; Make a menu bar from a list of MENU descriptions
  (let ((menu-bar (gtk-menu-bar-new)))
    (gtk-widget-show menu-bar)
    (for-each
     (lambda (menu) (gtk-menu-bar-append menu-bar (gtku-make-menu menu)))
     bar)
    menu-bar))

(define-syntax gtku-menu-item
  ;; (gtku-menu-item NAME VAR VAL [AFTER-SET] [*])
  ;; Simplify the construction of the ACTION callback for a menu-item.
  ;; Most menu callbacks set a variable to some fixed value
  ;; and then do probably some kind of redisplaying.
  ;;
  ;; More specifically, this macro creates a list (NAME . thunk)
  ;; where thunk is
  ;; (lambda () (if (not (eq? VAR VAL)) (begin (set! VAR VAL) AFTER-SET)))
  ;; VAR has to be a variable in the scope where gtku-menu-item is used
  ;; VAL is evaluated (only once) when gtku-menu-item is expanded
  ;; AFTER-SET is optional
  ;; If * is present, the expansion of the macro also performs a 
  ;; (set! VAR VAL). I.e. an item with a * becomes the default value
  (syntax-rules 
   (*)
   ((_ name var val *)
    (let ((v val))
      (set! var v)
      (gtku-menu-item name var v)))
   ((_ name var val after-set *)
    (let ((v val))
      (set! var v)
      (gtku-menu-item name var v after-set)))
   ((_ name var val)
    (let ((v val))
      (cons 'name (lambda () (if (not (eq? var v)) (set! var v))))))
   ((_ name var val after-set)
    (let ((v val))
      (cons 'name (lambda ()
		    (if (not (eq? var v)) (begin (set! var v) after-set))))))))

(define-syntax gtku-radio-menu
  ;; (gtku-radio-menu HEADING VAR (NAME VAL [AFTER-SET] [*])...)
  ;; creates a list denoting a radio menu
  ;; '(HEADING (radio (NAME . ACTION) ...))
  ;; where ACTION is constructed from VAL, AFTER-SET and * as described
  ;; for the macro gtku-menu-item
  (syntax-rules
   ()
   ((_ heading var (name val ...) ...)
    `(heading 
      (radio 
       . (,(gtku-menu-item name var val ...) ...))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yes or No dialog
;; (gtku-yes-or-no? QUESTION YES NO)
;; pops up a dialog box with the string QUESTION and two buttons,
;; one labelled with the string YES and one with the string NO
;; Returns #t if the user presses the YES button, #f in any other case
;; The dialog window is destroyed before returning
;;
;; From Marius Vollmers test-gtk.scm
(define (gtku-yes-or-no? question yes no)
  (let ((d (gtk-dialog-new))
	(l (gtk-label-new question))
	(y (gtk-button-new-with-label yes))
	(n (gtk-button-new-with-label no))

	(answer #f))
    (gtk-window-set-title d "Yes or No?")
    (gtk-misc-set-alignment l 0.5 0.5)
    (gtk-box-pack-start (gtk-dialog-vbox d) l #f #f 0)
    (gtk-box-pack-start (gtk-dialog-action-area d) y #t #t 3)
    (gtk-box-pack-start (gtk-dialog-action-area d) n #t #t 3)
    (gtku-widget-show* l y n d)

    (gtk-signal-connect d "delete_event" (lambda (ev) (gtk-main-quit) #t))
    (gtk-signal-connect n "clicked"
			(lambda ()
			  (gtk-widget-destroy d) 
			  (gtk-main-quit) #t))
    (gtk-signal-connect y "clicked"
			(lambda ()
			  (set! answer #t)
			  (gtk-widget-destroy d)
			  (gtk-main-quit)
			  #t))
    (gtk-grab-add d)
    (gtk-main)
    (gtk-grab-remove d)
    answer))

